@@ -11,35 +11,30 @@
 void lfModifier::AddColorCallback (
     lfModifyColorFunc callback, int priority, void *data, size_t data_size)
 {
-    lfExtModifier *This = static_cast<lfExtModifier *> (this);
     lfColorCallbackData *d = new lfColorCallbackData ();
     d->callback = callback;
-    This->AddCallback (This->ColorCallbacks, d, priority, data, data_size);
+    AddCallback (ColorCallbacks, d, priority, data, data_size);
 }
 
 bool lfModifier::AddColorCallbackVignetting (
     lfLensCalibVignetting &model, lfPixelFormat format, bool reverse)
 {
     float tmp [5];
-    lfExtModifier *This = static_cast<lfExtModifier *> (this);
 
-#define ADD_CALLBACK(func, format, type, prio) \
-    case LF_PF_ ## format: \
-        AddColorCallback ( \
-            (lfModifyColorFunc)(void (*)(void *, float, float, type *, int, int)) \
-            lfExtModifier::func, prio, tmp, 5 * sizeof (float)); \
-        break;
+#define ADD_CALLBACK(func, type, prio) \
+    AddColorCallback ( \
+        (lfModifyColorFunc)(void (*)(void *, float, float, type *, int, int)) \
+        lfModifier::func, prio, tmp, 5 * sizeof (float)) \
 
     memcpy (tmp, model.Terms, 3 * sizeof (float));
 
     // Damn! Hugin uses two different "normalized" coordinate systems:
-    // for distortions it uses 1.0 = max(half width, half height) and
+    // for distortions it uses 1.0 = min(half width, half height) and
     // for vignetting it uses 1.0 = half diagonal length. We have
-    // to compute a transition coefficient as we always work in
+    // to compute a transition coefficient as lfModifier works in
     // the first coordinate system.
-    double ns = 2.0 / sqrt (float (square (This->Width) + square (This->Height)));
-    tmp [3] = ns;
-    tmp [4] = ns / This->NormScale;
+    tmp [3] = NormScale / AspectRatioCorrection;
+    tmp [4] = 1.0 / AspectRatioCorrection;
 
     if (reverse)
         switch (model.Model)
@@ -47,11 +42,26 @@ bool lfModifier::AddColorCallbackVignetting (
             case LF_VIGNETTING_MODEL_PA:
                 switch (format)
                 {
-                    ADD_CALLBACK (ModifyColor_Vignetting_PA, U8, lf_u8, 250);
-                    ADD_CALLBACK (ModifyColor_Vignetting_PA, U16, lf_u16, 250);
-                    ADD_CALLBACK (ModifyColor_Vignetting_PA, U32, lf_u32, 250);
-                    ADD_CALLBACK (ModifyColor_Vignetting_PA, F32, lf_f32, 250);
-                    ADD_CALLBACK (ModifyColor_Vignetting_PA, F64, lf_f64, 250);
+                    case LF_PF_U8:
+                        ADD_CALLBACK (ModifyColor_Vignetting_PA, lf_u8, 250);
+                        break;
+
+                    case LF_PF_U16:
+                        ADD_CALLBACK (ModifyColor_Vignetting_PA, lf_u16, 250);
+                        break;
+
+                    case LF_PF_U32:
+                        ADD_CALLBACK (ModifyColor_Vignetting_PA, lf_u32, 250);
+                        break;
+
+                    case LF_PF_F32:
+                        ADD_CALLBACK (ModifyColor_Vignetting_PA, lf_f32, 250);
+                        break;
+
+                    case LF_PF_F64:
+                        ADD_CALLBACK (ModifyColor_Vignetting_PA, lf_f64, 250);
+                        break;
+
                     default:
                         return false;
                 }
@@ -66,11 +76,36 @@ bool lfModifier::AddColorCallbackVignetting (
             case LF_VIGNETTING_MODEL_PA:
                 switch (format)
                 {
-                    ADD_CALLBACK (ModifyColor_DeVignetting_PA, U8, lf_u8, 750);
-                    ADD_CALLBACK (ModifyColor_DeVignetting_PA, U16, lf_u16, 750);
-                    ADD_CALLBACK (ModifyColor_DeVignetting_PA, U32, lf_u32, 750);
-                    ADD_CALLBACK (ModifyColor_DeVignetting_PA, F32, lf_f32, 750);
-                    ADD_CALLBACK (ModifyColor_DeVignetting_PA, F64, lf_f64, 750);
+                    case LF_PF_U8:
+                        ADD_CALLBACK (ModifyColor_DeVignetting_PA, lf_u8, 750);
+                        break;
+
+                    case LF_PF_U16:
+#ifdef VECTORIZATION_SSE2
+                        if (_lf_detect_cpu_features () & LF_CPU_FLAG_SSE2)
+                            ADD_CALLBACK (ModifyColor_DeVignetting_PA_SSE2, lf_u16, 750);
+                        else
+#endif
+                        ADD_CALLBACK (ModifyColor_DeVignetting_PA, lf_u16, 750);
+                        break;
+
+                    case LF_PF_U32:
+                        ADD_CALLBACK (ModifyColor_DeVignetting_PA, lf_u32, 750);
+                        break;
+
+                    case LF_PF_F32:
+#ifdef VECTORIZATION_SSE
+                        if (_lf_detect_cpu_features () & LF_CPU_FLAG_SSE)
+                            ADD_CALLBACK (ModifyColor_DeVignetting_PA_SSE, lf_f32, 750);
+                        else
+#endif
+                        ADD_CALLBACK (ModifyColor_DeVignetting_PA, lf_f32, 750);
+                        break;
+
+                    case LF_PF_F64:
+                        ADD_CALLBACK (ModifyColor_DeVignetting_PA, lf_f64, 750);
+                        break;
+
                     default:
                         return false;
                 }
@@ -85,38 +120,21 @@ bool lfModifier::AddColorCallbackVignetting (
     return true;
 }
 
-/**
- * Uh-oh... I was not able to find the standard describing how exactly
- * CCI is computed or even what exactly it means. This is ISO 6728-83
- * or alternatively GOST R 50678-94 (Russian equivalent standard).
- * Perhaps even CCI might be removed from lensfun at one point in future
- * if there won't be enough interest in it and the standards information
- * couldn't be gathered.
- */
-bool lfModifier::AddColorCallbackCCI (
-    const lfLens *lens, lfPixelFormat format, bool reverse)
-{
-    (void)lens; (void)format; (void)reverse;
-    return false;
-}
-
 bool lfModifier::ApplyColorModification (
     void *pixels, float x, float y, int width, int height, int comp_role, int row_stride) const
 {
-    const lfExtModifier *This = static_cast<const lfExtModifier *> (this);
-
-    if (This->ColorCallbacks->len <= 0 || height <= 0)
+    if (((GPtrArray *)ColorCallbacks)->len <= 0 || height <= 0)
         return false; // nothing to do
 
-    x = x * This->NormScale - This->CenterX;
-    y = y * This->NormScale - This->CenterY;
+    x = x * NormScale - CenterX;
+    y = y * NormScale - CenterY;
 
-    for (; height; y += This->NormScale, height--)
+    for (; height; y += NormScale, height--)
     {
-        for (int i = 0; i < (int)This->ColorCallbacks->len; i++)
+        for (int i = 0; i < (int)((GPtrArray *)ColorCallbacks)->len; i++)
         {
             lfColorCallbackData *cd =
-                (lfColorCallbackData *)g_ptr_array_index (This->ColorCallbacks, i);
+                (lfColorCallbackData *)g_ptr_array_index ((GPtrArray *)ColorCallbacks, i);
             cd->callback (cd->data, x, y, pixels, comp_role, width);
         }
         pixels = ((char *)pixels) + row_stride;
@@ -162,11 +180,17 @@ template<typename T>static inline T *apply_multiplier (T *pixels, double c, int
     return pixels;
 }
 
+inline guint clampbits (gint x, guint n)
+{ guint32 _y_temp; if ((_y_temp = x >> n)) x = ~_y_temp >> (32 - n); return x; }
+
 // For lf_u8 pixel type do a more efficient arithmetic using fixed point
-template<>inline  lf_u8 *apply_multiplier (lf_u8 *pixels, double c, int &cr)
+template<>inline lf_u8 *apply_multiplier (lf_u8 *pixels, double c, int &cr)
 {
-    // Use 20.12 fixed-point math
+    // Use 20.12 fixed-point math. That leaves 11 bits (factor 2048)  as max multiplication
     int c12 = int (c * 4096.0);
+    if (c12 > (2047 << 12))
+        c12 = 2047 << 12;
+
     for (;;)
     {
         switch (cr & 15)
@@ -184,8 +208,8 @@ template<>inline  lf_u8 *apply_multiplier (lf_u8 *pixels, double c, int &cr)
             default:
                 if ((cr & 15) > LF_CR_UNKNOWN)
                 {
-                    int r = (int (*pixels) * c12) >> 12;
-                    *pixels = (r > 0xff) ? 0xff : ((r < 0) ? 0 : r);
+                    int r = (int (*pixels) * c12 + 2048) >> 12;
+                    *pixels = clampbits (r, 8);
                 }
                 break;
         }
@@ -195,7 +219,43 @@ template<>inline  lf_u8 *apply_multiplier (lf_u8 *pixels, double c, int &cr)
     return pixels;
 }
 
-template<typename T> void lfExtModifier::ModifyColor_Vignetting_PA (
+// For lf_u16 pixel type do a more efficient arithmetic using fixed point
+template<>inline lf_u16 *apply_multiplier (lf_u16 *pixels, double c, int &cr)
+{
+    // Use 22.10 fixed-point math. That leaves 6 bits (factor 32)  as max multiplication
+    int c10 = int (c * 1024.0);
+    if (c10 > (31 << 10))
+        c10 = 31 << 10;
+
+    for (;;)
+    {
+        switch (cr & 15)
+        {
+            case LF_CR_END:
+                return pixels;
+
+            case LF_CR_NEXT:
+                cr >>= 4;
+                return pixels;
+
+            case LF_CR_UNKNOWN:
+                break;
+
+            default:
+                if ((cr & 15) > LF_CR_UNKNOWN)
+                {
+                    int r = (int (*pixels) * c10 + 512) >> 10;
+                    *pixels = clampbits (r, 16);
+                }
+                break;
+        }
+        pixels++;
+        cr >>= 4;
+    }
+    return pixels;
+}
+
+template<typename T> void lfModifier::ModifyColor_Vignetting_PA (
     void *data, float x, float y, T *pixels, int comp_role, int count)
 {
     float *param = (float *)data;
@@ -209,16 +269,16 @@ template<typename T> void lfExtModifier::ModifyColor_Vignetting_PA (
     // But since we work in a normalized coordinate system, a step of
     // 1.0 pixels should be multiplied by NormScale, so it's really:
     // ((x+ns)*(x+ns)+y*y) - (x*x + y*y) = 2 * ns * x + ns^2
-    double r2 = x * x + y * y;
-    double d1 = 2.0 * param [3];
-    double d2 = param [3] * param [3];
+    float r2 = x * x + y * y;
+    float d1 = 2.0 * param [3];
+    float d2 = param [3] * param [3];
 
     int cr = 0;
     while (count--)
     {
-        double r4 = r2 * r2;
-        double r6 = r4 * r2;
-        double c = 1.0 + param [0] * r2 + param [1] * r4 + param [2] * r6;
+        float r4 = r2 * r2;
+        float r6 = r4 * r2;
+        float c = 1.0 + param [0] * r2 + param [1] * r4 + param [2] * r6;
         if (!cr)
             cr = comp_role;
 
@@ -229,7 +289,7 @@ template<typename T> void lfExtModifier::ModifyColor_Vignetting_PA (
     }
 }
 
-template<typename T> void lfExtModifier::ModifyColor_DeVignetting_PA (
+template<typename T> void lfModifier::ModifyColor_DeVignetting_PA (
     void *data, float x, float y, T *pixels, int comp_role, int count)
 {
     float *param = (float *)data;
@@ -237,20 +297,26 @@ template<typename T> void lfExtModifier::ModifyColor_DeVignetting_PA (
     x *= param [4];
     y *= param [4];
 
-    double r2 = x * x + y * y;
-    double d1 = 2.0 * param [3];
-    double d2 = param [3] * param [3];
+    // For faster computation we will compute r^2 here, and
+    // further compute just the delta:
+    // ((x+1)*(x+1)+y*y) - (x*x + y*y) = 2 * x + 1
+    // But since we work in a normalized coordinate system, a step of
+    // 1.0 pixels should be multiplied by NormScale, so it's really:
+    // ((x+ns)*(x+ns)+y*y) - (x*x + y*y) = 2 * ns * x + ns^2
+    float r2 = x * x + y * y;
+    float d1 = 2.0 * param [3];
+    float d2 = param [3] * param [3];
 
     int cr = 0;
     while (count--)
     {
-        double r4 = r2 * r2;
-        double r6 = r4 * r2;
-        double c = 1.0 + param [0] * r2 + param [1] * r4 + param [2] * r6;
+        float r4 = r2 * r2;
+        float r6 = r4 * r2;
+        float c = 1.0 + param [0] * r2 + param [1] * r4 + param [2] * r6;
         if (!cr)
             cr = comp_role;
 
-        pixels = apply_multiplier<T> (pixels, 1.0 / c, cr);
+        pixels = apply_multiplier<T> (pixels, 1.0f / c, cr);
 
         r2 += d1 * x + d2;
         x += param [3];
@@ -273,13 +339,6 @@ cbool lf_modifier_add_color_callback_vignetting (
     return modifier->AddColorCallbackVignetting (*model, format, reverse);
 }
 
-cbool lf_modifier_add_color_callback_CCI (
-    lfModifier *modifier, const lfLens *lens,
-    lfPixelFormat format, cbool reverse)
-{
-    return modifier->AddColorCallbackCCI (lens, format, reverse);
-}
-
 cbool lf_modifier_apply_color_modification (
     lfModifier *modifier, void *pixels, float x, float y, int width, int height,
     int comp_role, int row_stride)
