@@ -11,10 +11,9 @@
 void lfModifier::AddSubpixelCallback (
     lfSubpixelCoordFunc callback, int priority, void *data, size_t data_size)
 {
-    lfExtModifier *This = static_cast<lfExtModifier *> (this);
     lfSubpixelCallbackData *d = new lfSubpixelCallbackData ();
     d->callback = callback;
-    This->AddCallback (This->SubpixelCallbacks, d, priority, data, data_size);
+    AddCallback (SubpixelCallbacks, d, priority, data, data_size);
 }
 
 bool lfModifier::AddSubpixelCallbackTCA (lfLensCalibTCA &model, bool reverse)
@@ -34,12 +33,12 @@ bool lfModifier::AddSubpixelCallbackTCA (lfLensCalibTCA &model, bool reverse)
                         return false;
                     tmp [i] = 1.0 / model.Terms [i];
                 }
-                AddSubpixelCallback (lfExtModifier::ModifyCoord_TCA_Linear, 500,
+                AddSubpixelCallback (ModifyCoord_UnTCA_Linear, 500,
                                      tmp, 2 * sizeof (float));
                 return true;
 
             case LF_TCA_MODEL_POLY3:
-                AddSubpixelCallback (lfExtModifier::ModifyCoord_TCA_Poly3, 500,
+                AddSubpixelCallback (ModifyCoord_UnTCA_Poly3, 500,
                                      model.Terms, 6 * sizeof (float));
                 return true;
 
@@ -54,12 +53,12 @@ bool lfModifier::AddSubpixelCallbackTCA (lfLensCalibTCA &model, bool reverse)
                 break;
 
             case LF_TCA_MODEL_LINEAR:
-                AddSubpixelCallback (lfExtModifier::ModifyCoord_UnTCA_Linear, 500,
+                AddSubpixelCallback (ModifyCoord_TCA_Linear, 500,
                                      model.Terms, 2 * sizeof (float));
                 return true;
 
             case LF_TCA_MODEL_POLY3:
-                AddSubpixelCallback (lfExtModifier::ModifyCoord_UnTCA_Poly3, 500,
+                AddSubpixelCallback (ModifyCoord_TCA_Poly3, 500,
                                      model.Terms, 6 * sizeof (float));
                 return true;
 
@@ -74,39 +73,37 @@ bool lfModifier::AddSubpixelCallbackTCA (lfLensCalibTCA &model, bool reverse)
 bool lfModifier::ApplySubpixelDistortion (
     float xu, float yu, int width, int height, float *res) const
 {
-    const lfExtModifier *This = static_cast<const lfExtModifier *> (this);
-
-    if (This->SubpixelCallbacks->len <= 0 || height <= 0)
+    if (((GPtrArray *)SubpixelCallbacks)->len <= 0 || height <= 0)
         return false; // nothing to do
 
     // All callbacks work with normalized coordinates
-    xu = xu * This->NormScale - This->CenterX;
-    yu = yu * This->NormScale - This->CenterY;
+    xu = xu * NormScale - CenterX;
+    yu = yu * NormScale - CenterY;
 
-    for (float y = yu; height; y += This->NormScale, height--)
+    for (float y = yu; height; y += NormScale, height--)
     {
         int i;
         float x = xu;
         float *out = res;
-        for (i = 0; i < width; i++, x += This->NormScale)
+        for (i = 0; i < width; i++, x += NormScale)
         {
             out [0] = out [2] = out [4] = x;
             out [1] = out [3] = out [5] = y;
             out += 6;
         }
 
-        for (i = 0; i < (int)This->SubpixelCallbacks->len; i++)
+        for (i = 0; i < (int)((GPtrArray *)SubpixelCallbacks)->len; i++)
         {
             lfSubpixelCallbackData *cd =
-                (lfSubpixelCallbackData *)g_ptr_array_index (This->SubpixelCallbacks, i);
+                (lfSubpixelCallbackData *)g_ptr_array_index ((GPtrArray *)SubpixelCallbacks, i);
             cd->callback (cd->data, res, width);
         }
 
         // Convert normalized coordinates back into natural coordiates
         for (i = width * 3; i > 0; i--)
         {
-            res [0] = (res [0] + This->CenterX) * This->NormUnScale;
-            res [1] = (res [1] + This->CenterY) * This->NormUnScale;
+            res [0] = (res [0] + CenterX) * NormUnScale;
+            res [1] = (res [1] + CenterY) * NormUnScale;
             res += 2;
         }
     }
@@ -117,47 +114,45 @@ bool lfModifier::ApplySubpixelDistortion (
 bool lfModifier::ApplySubpixelGeometryDistortion (
     float xu, float yu, int width, int height, float *res) const
 {
-    const lfExtModifier *This = static_cast<const lfExtModifier *> (this);
-
-    if ((This->SubpixelCallbacks->len <= 0 && This->CoordCallbacks->len <= 0)
+    if ((((GPtrArray *)SubpixelCallbacks)->len <= 0 && ((GPtrArray *)CoordCallbacks)->len <= 0)
      || height <= 0)
         return false; // nothing to do
 
     // All callbacks work with normalized coordinates
-    xu = xu * This->NormScale - This->CenterX;
-    yu = yu * This->NormScale - This->CenterY;
+    xu = xu * NormScale - CenterX;
+    yu = yu * NormScale - CenterY;
 
-    for (float y = yu; height; y += This->NormScale, height--)
+    for (float y = yu; height; y += NormScale, height--)
     {
         int i;
         float x = xu;
         float *out = res;
-        for (i = 0; i < width; i++, x += This->NormScale)
+        for (i = 0; i < width; i++, x += NormScale)
         {
             out [0] = out [2] = out [4] = x;
             out [1] = out [3] = out [5] = y;
             out += 6;
         }
 
-        for (i = 0; i < (int)This->SubpixelCallbacks->len; i++)
-        {
-            lfSubpixelCallbackData *cd =
-                (lfSubpixelCallbackData *)g_ptr_array_index (This->SubpixelCallbacks, i);
-            cd->callback (cd->data, res, width);
-        }
-
-        for (i = 0; i < (int)This->CoordCallbacks->len; i++)
+        for (i = 0; i < (int)((GPtrArray *)CoordCallbacks)->len; i++)
         {
             lfCoordCallbackData *cd =
-                (lfCoordCallbackData *)g_ptr_array_index (This->CoordCallbacks, i);
+                (lfCoordCallbackData *)g_ptr_array_index ((GPtrArray *)CoordCallbacks, i);
             cd->callback (cd->data, res, width * 3);
         }
 
+        for (i = 0; i < (int)((GPtrArray *)SubpixelCallbacks)->len; i++)
+        {
+            lfSubpixelCallbackData *cd =
+                (lfSubpixelCallbackData *)g_ptr_array_index ((GPtrArray *)SubpixelCallbacks, i);
+            cd->callback (cd->data, res, width);
+        }
+
         // Convert normalized coordinates back into natural coordiates
         for (i = width * 3; i > 0; i--)
         {
-            res [0] = (res [0] + This->CenterX) * This->NormUnScale;
-            res [1] = (res [1] + This->CenterY) * This->NormUnScale;
+            res [0] = (res [0] + CenterX) * NormUnScale;
+            res [1] = (res [1] + CenterY) * NormUnScale;
             res += 2;
         }
     }
@@ -165,7 +160,7 @@ bool lfModifier::ApplySubpixelGeometryDistortion (
     return true;
 }
 
-void lfExtModifier::ModifyCoord_TCA_Linear (void *data, float *iocoord, int count)
+void lfModifier::ModifyCoord_UnTCA_Linear (void *data, float *iocoord, int count)
 {
     float *param = (float *)data;
     float k_r = param [0];
@@ -180,7 +175,7 @@ void lfExtModifier::ModifyCoord_TCA_Linear (void *data, float *iocoord, int coun
     }
 }
 
-void lfExtModifier::ModifyCoord_UnTCA_Linear (void *data, float *iocoord, int count)
+void lfModifier::ModifyCoord_TCA_Linear (void *data, float *iocoord, int count)
 {
     float *param = (float *)data;
     float k_r = param [0];
@@ -195,7 +190,7 @@ void lfExtModifier::ModifyCoord_UnTCA_Linear (void *data, float *iocoord, int co
     }
 }
 
-void lfExtModifier::ModifyCoord_TCA_Poly3 (void *data, float *iocoord, int count)
+void lfModifier::ModifyCoord_UnTCA_Poly3 (void *data, float *iocoord, int count)
 {
     const float *param = (float *)data;
     const float vr = param [0];
@@ -208,75 +203,75 @@ void lfExtModifier::ModifyCoord_TCA_Poly3 (void *data, float *iocoord, int count
     for (float *end = iocoord + count * 2 * 3; iocoord < end; iocoord += 6)
     {
         float x, y;
-        double ru, rd, rd2;
+        double rd, ru, ru2;
 
         x = iocoord [0];
         y = iocoord [1];
-        ru = sqrt (x * x + y * y);
-        if (ru == 0.0)
+        rd = sqrt (x * x + y * y);
+        if (rd == 0.0)
             goto next_subpixel_r;
 
-        // Original equation: Ru = b * Rd^3 + c * Rd^2 + v * Rd
-        // We have Ru, need to find Rd. We will use Newton's method
+        // Original equation: Rd = b * Ru^3 + c * Ru^2 + v * Ru
+        // We have Rd, need to find Ru. We will use Newton's method
         // for finding the root of the equation:
         //
-        // Target function: b * Rd^3 + c * Rd^2 + v * Rd - Ru = 0
-        // Derivative:      3 * b * Rd^2 + 2 * c * Rd + v
-        rd = ru;
+        // Target function: b * Ru^3 + c * Ru^2 + v * Ru - Rd = 0
+        // Derivative:      3 * b * Ru^2 + 2 * c * Ru + v
+        ru = rd;
         for (int step = 0; ; step++)
         {
-            rd2 = rd * rd;
-            double frd = br * rd2 * rd + cr * rd2 + vr * rd - ru;
-            if (frd >= -NEWTON_EPS && frd < NEWTON_EPS)
+            ru2 = ru * ru;
+            double fru = br * ru2 * ru + cr * ru2 + vr * ru - rd;
+            if (fru >= -NEWTON_EPS && fru < NEWTON_EPS)
                 break;
             if (step > 5)
                 // Does not converge, no real solution in this area?
                 goto next_subpixel_r;
 
-            rd -= frd / (3 * br * rd2 + 2 * cr * rd + vr);
+            ru -= fru / (3 * br * ru2 + 2 * cr * ru + vr);
         }
         // Negative radius does not make sense at all
-        if (rd > 0.0)
+        if (ru > 0.0)
         {
-            rd /= ru;
-            iocoord [0] = x * rd;
-            iocoord [1] = y * rd;
+            ru /= rd;
+            iocoord [0] = x * ru;
+            iocoord [1] = y * ru;
         }
 next_subpixel_r:
 
         x = iocoord [4];
         y = iocoord [5];
-        ru = sqrt (x * x + y * y);
-        if (ru == 0.0)
+        rd = sqrt (x * x + y * y);
+        if (rd == 0.0)
             goto next_subpixel_b;
 
-        rd = ru;
+        ru = rd;
         for (int step = 0; ; step++)
         {
-            rd2 = rd * rd;
-            double frd = bb * rd2 * rd + cb * rd2 + vb * rd - ru;
-            if (frd >= -NEWTON_EPS && frd < NEWTON_EPS)
+            ru2 = ru * ru;
+            double fru = bb * ru2 * ru + cb * ru2 + vb * ru - rd;
+            if (fru >= -NEWTON_EPS && fru < NEWTON_EPS)
                 break;
             if (step > 5)
                 // Does not converge, no real solution in this area?
                 goto next_subpixel_b;
 
-            rd -= frd / (3 * bb * rd2 + 2 * cb * rd + vb);
+            ru -= fru / (3 * bb * ru2 + 2 * cb * ru + vb);
         }
         // Negative radius does not make sense at all
-        if (rd > 0.0)
+        if (ru > 0.0)
         {
-            rd /= ru;
-            iocoord [4] = x * rd;
-            iocoord [5] = y * rd;
+            ru /= rd;
+            iocoord [4] = x * ru;
+            iocoord [5] = y * ru;
         }
 next_subpixel_b:;
     }
 }
 
-void lfExtModifier::ModifyCoord_UnTCA_Poly3 (void *data, float *iocoord, int count)
+void lfModifier::ModifyCoord_TCA_Poly3 (void *data, float *iocoord, int count)
 {
-    // Ru = Rd * (b * Rd^2 + c * Rd + v)
+    // Rd = Ru * (b * Ru^2 + c * Ru + v)
     const float *param = (float *)data;
     const float vr = param [0];
     const float vb = param [1];
@@ -285,22 +280,22 @@ void lfExtModifier::ModifyCoord_UnTCA_Poly3 (void *data, float *iocoord, int cou
     const float br = param [4];
     const float bb = param [5];
 
-    float x, y, r2, poly2;
+    float x, y, ru2, poly2;
     // Optimize for the case when c == 0 (avoid two square roots per pixel)
     if (cr == 0.0 && cb == 0.0)
         for (float *end = iocoord + count * 2 * 3; iocoord < end; iocoord += 6)
         {
             x = iocoord [0];
             y = iocoord [1];
-            r2 = x * x + y * y;
-            poly2 = br * r2 + vr;
+            ru2 = x * x + y * y;
+            poly2 = br * ru2 + vr;
             iocoord [0] = x * poly2;
             iocoord [1] = y * poly2;
 
             x = iocoord [4];
             y = iocoord [5];
-            r2 = x * x + y * y;
-            poly2 = bb * r2 + vb;
+            ru2 = x * x + y * y;
+            poly2 = bb * ru2 + vb;
             iocoord [4] = x * poly2;
             iocoord [5] = y * poly2;
         }
@@ -309,15 +304,15 @@ void lfExtModifier::ModifyCoord_UnTCA_Poly3 (void *data, float *iocoord, int cou
         {
             x = iocoord [0];
             y = iocoord [1];
-            r2 = x * x + y * y;
-            poly2 = br * r2 + cr * sqrt (r2) + vr;
+            ru2 = x * x + y * y;
+            poly2 = br * ru2 + cr * sqrt (ru2) + vr;
             iocoord [0] = x * poly2;
             iocoord [1] = y * poly2;
 
             x = iocoord [4];
             y = iocoord [5];
-            r2 = x * x + y * y;
-            poly2 = bb * r2 + cb * sqrt (r2) + vb;
+            ru2 = x * x + y * y;
+            poly2 = bb * ru2 + cb * sqrt (ru2) + vb;
             iocoord [4] = x * poly2;
             iocoord [5] = y * poly2;
         }
